<?php

namespace Drupal\xnttyaml\Plugin\ExternalEntities\StorageClient;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\external_entities\ResponseDecoder\ResponseDecoderFactoryInterface;
use Drupal\xnttfiles\Plugin\ExternalEntities\StorageClient\FileClientBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\Yaml\Yaml;
use Symfony\Component\Yaml\Exception\ParseException;

/**
 * External entities storage client for Yaml files.
 *
 * @ExternalEntityStorageClient(
 *   id = "xnttyaml",
 *   label = @Translation("Yaml Files"),
 *   description = @Translation("Retrieves and stores records in Yaml files.")
 * )
 */
class YamlFiles extends FileClientBase {

  /**
   * {@inheritdoc}
   */
  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    TranslationInterface $string_translation,
    ResponseDecoderFactoryInterface $response_decoder_factory,
    ConfigFactory $config_factory,
    MessengerInterface $messenger,
    LoggerChannelInterface $logger,
    CacheBackendInterface $cache
  ) {
    parent::__construct(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $string_translation,
      $response_decoder_factory,
      $config_factory,
      $messenger,
      $logger,
      $cache
    );
    // Defaults.
    $this->fileType = 'yaml';
    $this->fileTypePlural = 'Yaml';
    $this->fileTypeCap = 'Yaml';
    $this->fileTypeCapPlural = 'Yaml';
    $this->fileExtension = '.yml';
  }

  /**
   * {@inheritdoc}
   */
  public static function create(
    ContainerInterface $container,
    array $configuration,
    $plugin_id,
    $plugin_definition
  ) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('string_translation'),
      $container->get('external_entities.response_decoder_factory'),
      $container->get('config.factory'),
      $container->get('messenger'),
      $container->get('logger.channel.xnttyaml'),
      $container->get('cache.default')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function parseFile(string $file_path) :array {
    $data = [];
    $idf = $this->getIdField();
    try {
      $yaml = Yaml::parseFile($file_path);
      if ($this->configuration['multi_records']) {
        $columns = [];
          // Multiple records, we assume we have an array.
          if (is_array($yaml)) {
            // Process each entity.
            foreach ($yaml as $entity) {
              if (!empty($entity[$idf])) {
                $data[$entity[$idf]] = $entity;
              }
              $columns += array_flip(array_keys($entity));
            }
          }
          $data[''] = array_keys($columns);
      }
      else {
          // Single record. We assume we have one entity.
          if (is_array($yaml) && (!empty($yaml[$idf]))) {
            $data[$yaml[$idf]] = $yaml;
            $data[''] = array_keys($yaml);
          }
      }
    }
    catch (ParseException $e) {
      $this->logger->error(
        'Failed to parse Yaml file "'
        . $file_path
        . '" near line '
        . $e->getParsedLine()
        . ': '
        . $e->getSnippet()
      );
    }
    return $data;
  }

  /**
   * {@inheritdoc}
   */
  protected function generateRawData(array $entities_data) :string {
    // Don't save columns.
    $columns = array_flip($entities_data['']);
    unset($entities_data['']);
    if ($this->configuration['multi_records']) {
      // Multiple values.
      $yaml_entities = [];
      foreach ($entities_data as $id => $entity) {
        $yaml_entities[] = array_intersect_key($entity, $columns);
      }
      $yaml_raw = Yaml::dump($yaml_entities);
    }
    else {
      // Single value.
      $entity = array_shift($entities_data);
      $yaml_entity = array_intersect_key($entity, $columns);
      $yaml_raw = Yaml::dump($yaml_entity);
    }
    // Write records.
    return $yaml_raw;
  }

}
